/* 
   cadaver, command-line DAV client
   Copyright (C) 1999-2000, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef CADAVER_H
#define CADAVER_H

#ifdef HAVE_LIBREADLINE

/* readline requires FILE *, silly thing */
#include <stdio.h>

#ifdef HAVE_READLINE_H
#include <readline.h>
#elif HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#endif

#ifdef HAVE_HISTORY_H
#include <history.h>
#elif HAVE_READLINE_HISTORY_H
#include <readline/history.h>
#endif

#endif

#include "common.h"

/* DON'T FORGET TO ADD A NEW COMMAND ALIAS WHEN YOU ADD A NEW COMMAND */
enum command_id {
    cmd_ls, cmd_cd, cmd_quit, cmd_open, cmd_close, cmd_version, cmd_pwd,
    cmd_help, cmd_put, cmd_get, cmd_mkcol, cmd_delete, cmd_move, cmd_copy, 
    cmd_less, cmd_cat, cmd_lpwd, cmd_lcd, cmd_lls, cmd_mput, cmd_mget,
    cmd_echo, cmd_set, cmd_unset, cmd_rmcol, cmd_lock, cmd_unlock,
    cmd_steal, cmd_discover, cmd_showlocks, cmd_propedit, cmd_propnames,
    cmd_edit, 
    cmd_unknown
/* DON'T FORGET TO ADD A NEW COMMAND ALIAS WHEN YOU ADD A NEW COMMAND */
};

struct command {
    enum command_id id; /* unique ID */
    bool needs_connection; /* true if only works when connected */
    int min_args, max_args; /* min and max # of arguments */
    enum { /* parameter scope */
	parmscope_none, /* not at all */
	parmscope_option, /* option parameter */
	parmscope_local, /* local filenames */
	parmscope_remote /* remote filename */
    } scope;
    void *handler; /* the handler function */
    const char *call, /* command usage */
	*short_help; /* single-line help message */
};

/* The command array */
extern const struct command commands[];

extern struct proto_host server;

extern bool have_connection, dav_collection;

extern char *path, /* current working collection */
    *old_path; /* previous working collection */

/* Sets the current collection to the given path.  Returns zero on
 * success, non-zero if newpath is an untolerated non-WebDAV
 * collection. */
int set_path( const char *newpath );

extern int tolerant;

extern int in_curses;

void init_signals( void );

#ifdef HAVE_LIBREADLINE
char *command_generator( char *text, int state );
#else
/* roll our own */
char *readline( const char *prompt );
#endif

/* Command feedback handling */
enum output_type {
    o_start,
    o_transfer,
    o_finish
};

void output( enum output_type, const char *fmt, ... )
#ifdef __GNUC__
                __attribute__ ((format (printf, 2, 3)))
#endif /* __GNUC__ */
;

#endif /* CADAVER_H */
